import pytest

from lazyfpl import helpers, structures

# Sample Player instances for testing with necessary attributes
player1 = structures.Player(
    fixutres=[],
    name="Player1",
    news="",
    position="GKP",
    price=25,
    selected=11,
    team="TeamA",
    team_short="TA",
    webname="Player1",
    xP=1.0,
)
player2 = structures.Player(
    fixutres=[],
    name="Player2",
    news="",
    position="DEF",
    price=50,
    selected=22,
    team="TeamA",
    team_short="TA",
    webname="Player2",
    xP=2.0,
)
player3 = structures.Player(
    fixutres=[],
    name="Player3",
    news="",
    position="MID",
    price=75,
    selected=33,
    team="TeamB",
    team_short="TB",
    webname="Player3",
    xP=3.0,
)
player4 = structures.Player(
    fixutres=[],
    name="Player4",
    news="",
    position="FWD",
    price=100,
    selected=44,
    team="TeamC",
    team_short="TC",
    webname="Player4",
    xP=4.0,
)
lineup = [player1, player2, player3, player4]


@pytest.mark.parametrize(
    "lineup, expected_price",
    [
        (lineup, 250),
        ([], 0),
    ],
)
def test_squad_price(
    lineup: list[structures.Player],
    expected_price: int,
) -> None:
    assert helpers.squad_price(lineup) == expected_price


@pytest.mark.parametrize(
    "lineup, expected_xp",
    [
        (lineup, 10.0),
        ([], 0.0),
    ],
)
def test_squad_xP(
    lineup: list[structures.Player],
    expected_xp: float,
) -> None:
    assert helpers.squad_xP(lineup) == expected_xp


@pytest.mark.parametrize(
    "lineup, expected_overall_xp",
    [
        (lineup, pytest.approx(14.1421, 0.0001)),
        ([], 0.0),
    ],
)
def test_overall_xP(
    lineup: list[structures.Player],
    expected_overall_xp: float,
) -> None:
    assert pytest.approx(helpers.overall_xP(lineup)) == expected_overall_xp



from __future__ import annotations

import pytest

from lazyfpl import constraints, structures

# Sample Player instances for testing
player1 = structures.Player(
    fixutres=[],
    name="Player1",
    news="",
    position="GKP",
    price=100,
    selected=11,
    team="TeamA",
    team_short="TA",
    webname="Player1",
    xP=None,
)
player2 = structures.Player(
    fixutres=[],
    name="Player2",
    news="",
    position="DEF",
    price=100,
    selected=22,
    team="TeamA",
    team_short="TA",
    webname="Player2",
    xP=None,
)
player3 = structures.Player(
    fixutres=[],
    name="Player3",
    news="",
    position="MID",
    price=100,
    selected=33,
    team="TeamB",
    team_short="TB",
    webname="Player3",
    xP=None,
)
player4 = structures.Player(
    fixutres=[],
    name="Player4",
    news="",
    position="FWD",
    price=100,
    selected=44,
    team="TeamC",
    team_short="TC",
    webname="Player4",
    xP=None,
)


@pytest.mark.parametrize(
    "lineup, n, expected",
    [
        ([player1, player2, player3], 2, True),
        ([player1, player1, player3], 1, False),
    ],
)
def test_team_constraint(
    lineup: list[structures.Player],
    n: int,
    expected: bool,
) -> None:
    assert constraints.team_constraint(lineup, n) == expected


@pytest.mark.parametrize(
    "lineup, expected",
    [
        (
            [player1, player2, player3],
            True,
        ),
        (
            [player1, player3, player4],
            False,
        ),
    ],
)
def test_gkp_def_same_team(
    lineup: list[structures.Player],
    expected: bool,
) -> None:
    assert constraints.gkp_def_same_team(lineup) == expected



# ruff: noqa: E501
from __future__ import annotations

import functools
import pathlib
import sqlite3

from lazyfpl import conf, structures


@functools.cache
def connect(file: pathlib.Path = conf.db) -> sqlite3.Connection:
    """Establishes a SQLite database connection using the provided file path."""
    return sqlite3.connect(file)


def execute(sql: str, parameters: tuple = ()) -> list[dict]:
    """Executes a SQL query and returns the result as a list of dictionaries."""
    with connect() as connection:
        cursor = connection.execute(sql, parameters)
        if desc := [x[0] for x in cursor.description or []]:
            return [dict(zip(desc, row)) for row in cursor.fetchall()]
        return []


def executemany(sql: str, parameters: tuple = ()) -> list[dict]:
    """Executes a SQL query and returns the result as a list of dictionaries."""
    with connect() as connection:
        cursor = connection.executemany(sql, parameters)
        if desc := [x[0] for x in cursor.description or []]:
            return [dict(zip(desc, row)) for row in cursor.fetchall()]
        return []


@functools.cache
def games() -> list[structures.Game]:
    """Retrieves a list of Game objects representing football games from the database."""
    rows = execute(
        """
        SELECT
            g.gw,
            g.is_home,
            g.kickoff,
            g.minutes,
            g.player_id,
            g.points,
            g.position,
            g.selected,
            g.session,
            g.upcoming,
            p.name            AS player,
            p.news            AS news,
            p.webname         AS webname,
            t.name            AS team,
            t.strength        AS team_strength,
            t.short_name      AS team_short,
            opp.name          AS opponent,
            opp.strength      AS opponent_strength,
            opp.short_name    AS opponent_short
        FROM
            game g
        JOIN
            player p ON p.id = g.player_id
        JOIN
            team t ON t.id = g.team
        JOIN
            team opp ON opp.id = g.opponent
    """
    )

    return [structures.Game.model_validate(row) for row in rows]


def price(pid: int) -> int:
    """Fetches and returns the price of a player based on their ID."""
    rows = execute(
        """
        SELECT
            price
        FROM
            player
        WHERE
            id = ?
    """,
        (pid,),
    )
    return rows[0]["price"]


def webname(pid: int) -> str:
    """Retrieves and returns the web name of a player based on their ID."""
    rows = execute(
        """
        SELECT
            webname
        FROM
            player
        WHERE
            id = ?
    """,
        (pid,),
    )
    return rows[0]["webname"]


def save_model(player_id: int, model: bytes) -> None:
    """Saves a machine learning model to the database for a given player ID."""
    execute(
        """
        UPDATE
            player
        SET
            model = ?
        WHERE
            id = ?
    """,
        (model, player_id),
    )


def load_model(player_id: int) -> bytes:
    """Loads and returns a machine learning model from the database for a given player ID."""
    return execute(
        """
        SELECT
            model
        FROM
            player
        WHERE
            id =?
    """,
        (player_id,),
    )[0]["model"]


@functools.cache
def points() -> structures.Summary:
    """SampleSummary of points scored in games."""
    return structures.Summary.fromiter(
        [
            row["points"]
            for row in execute(
                """
        SELECT
            points
        FROM
            game
        WHERE
            points is not null
    """
            )
        ]
    )


@functools.cache
def minutes() -> structures.Summary:
    """SampleSummary of minutes played in games."""
    return structures.Summary.fromiter(
        [
            row["minutes"]
            for row in execute(
                """
        SELECT
            minutes
        FROM
            game
        WHERE
            minutes is not null
    """
            )
        ]
    )
